# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, Field, StrictBytes, StrictStr
from typing_extensions import Annotated, Self

from root.generated.openapi_client.models.data_set_type import DataSetType


class DataSetCreateRequest(BaseModel):
    """
    DataSetCreateRequest
    """  # noqa: E501

    name: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    file: Optional[Union[StrictBytes, StrictStr]] = None
    draft_id: Optional[StrictStr] = None
    type: Optional[DataSetType] = None
    url: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    tags: Optional[List[StrictStr]] = None
    __properties: ClassVar[List[str]] = ["name", "file", "draft_id", "type", "url", "tags"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DataSetCreateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DataSetCreateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "file": obj.get("file"),
                "draft_id": obj.get("draft_id"),
                "type": obj.get("type"),
                "url": obj.get("url"),
                "tags": obj.get("tags"),
            }
        )
        return _obj



import tempfile

from root import RootSignals
from root.skills import ReferenceVariable
from root.validators import Validator

client = RootSignals()

with tempfile.NamedTemporaryFile(suffix=".csv", mode="w") as fp:
    fp.write("short@example.com\nlonger-email@example.com\n")
    fp.flush()
    dataset = client.datasets.create(
        name="List of email addresses", path=fp.name, type="reference"
    )

skill = client.skills.create(
    reference_variables=[
        ReferenceVariable(dataset_id=dataset.id, name="email_dataset")
    ],
    intent="Email address list assistant.",
    name="Email address dataset chatbot",
    prompt="{{email_dataset}}",
    model="gpt-4",
    validators=[
        Validator(
            evaluator_name="Email address checker",
            prompt="Is it an email address?",
            threshold=0.6,
        )
    ],
)

response = skill.run(
    {
        "email_dataset": (
            "Which email has the longest domain name?"
            "Respond with just the email address."
        )
    },
)
print(response.llm_output)
# longer-email@example.com
print(response.validation)
# {'validator_results': [{'cost': 0.000608, 'evaluator_name': 'Email
# address checker', 'threshold': 0.6,
# 'is_valid': True, 'result': 1.0, 'status': 'finished'}], 'is_valid':
# True}

# Run it with something that is not related to the dataset
blocked_response = skill.run({"email_dataset": "How can I cook Korean food?"})
# See that the response is blocked
print(blocked_response.llm_output)
# This message is not available.
print(blocked_response.validation.is_valid)
# False



from functools import partial
from typing import Iterator, Optional

from root.generated.openapi_client.api.models_api import ModelsApi
from root.generated.openapi_client.models.model import Model
from root.generated.openapi_client.models.model_create_request import ModelCreateRequest

from .generated.openapi_client import ApiClient
from .utils import iterate_cursor_list


class Models:
    """Models (sub) API

    Note:

      The construction of the API instance should be handled by
      accesing an attribute of a :class:`root.client.RootSignals` instance.
    """

    def __init__(self, client: ApiClient):
        self.client = client

    def list(self, *, limit: int = 100) -> Iterator[Model]:
        """Iterate through the models.

        Note:

          The call will list only publicly available global models and
          those models available to the organzation(s) of the user.

        Args:
          limit: Number of entries to iterate through at most.

        """
        api_instance = ModelsApi(self.client)
        yield from iterate_cursor_list(partial(api_instance.models_list), limit=limit)

    def create(
        self,
        *,
        name: str,
        model: Optional[str] = None,
        default_key: Optional[str] = None,
        max_output_token_count: Optional[int] = None,
        max_token_count: Optional[int] = None,
        url: Optional[str] = None,
    ) -> str:
        """Create a new model and return its ID.

        Args:

          name: The unique identifier for the model instance  (e.g. "google/gemma-2-9b").

          model: The base model name to be used. Defaults to name.

          default_key: The default API key required for the model, if applicable.

          max_output_token_count: The maximum number of tokens to output.

          max_token_count: The maximum number of tokens to process.

          url: Optional URL pointing to the model's endpoint.

        """
        request = ModelCreateRequest(
            name=name,
            model=model,
            default_key=default_key,
            max_output_token_count=max_output_token_count,
            max_token_count=max_token_count,
            is_local=url is not None,
            url=url,
        )

        api_instance = ModelsApi(self.client)
        return api_instance.models_create(model_create_request=request).id

    def delete(self, model_id: str) -> None:
        """
        Delete the model.

        Args:

          model: The model to be deleted.

        """
        api_instance = ModelsApi(self.client)
        return api_instance.models_destroy(id=model_id)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "root-signals Python SDK"
copyright = "2024, Root Signals Ltd"  # noqa: A001
author = "Root Signals"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "autoapi.extension",
    "myst_parser",  # markdown support as input
    "sphinx.ext.autodoc",  # for autoapi
    "sphinx.ext.doctest",  # automated test running
    "sphinx.ext.inheritance_diagram",  # for autoapi
    "sphinx.ext.napoleon",  # nicer syntax for docstrings - see https://www.sphinx-doc.org/en/master/usage/extensions/napoleon.html
    "sphinx_markdown_builder",
]

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

# html_theme = "alabaster"
html_theme = "furo"
# html_static_path = ["_static"]

autodoc_typehints = "both"  # default is 'signature' (not in description)

autoapi_dirs = ["../src/root"]
autoapi_ignore = ["*/utils.py"]

autoapi_options = [
    # "imported-members", # this is per same top level, and we wind up with cruft from generated code
    # 'inherited-members', # frequently pointless
    # 'private-members', # they should not be used -> omit
    # 'show-inheritance-diagram', # relatively ugly
    # 'special-members',
    "members",
    "show-inheritance",
    "show-module-summary",
    "undoc-members",
]

autoapi_member_order = "groupwise"

# Useful for debugging
# autoapi_keep_files = True

# suppress_warnings = ["autoapi"]


def skip_unwanted(app, what, name, obj, skip, options):
    # what: "attribute", "class", "data", "exception", "function", "method", "module", "package"
    # name: fully qualified name of obj
    # obj: object itself

    if ".generated." in name:
        # We only refer to models
        if what == "module" and ".models." not in name:
            return True
        # Our API should hide the actual request objects
        if what == "module" and name.endswith("_request"):
            return True
        # Omit the methods from generated code.
        # In general there's plenty of from_X, to_X for all generated models which are simply not relevant.
        if what == "method":
            return True

    return skip


def setup(sphinx):
    sphinx.connect("autoapi-skip-member", skip_unwanted)



from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from root.generated.openapi_client.models.skill_validator_request import SkillValidatorRequest

if TYPE_CHECKING:
    from .skills import ModelName, Skills


class Validator:
    def __init__(
        self,
        *,
        evaluator_id: Optional[str] = None,
        evaluator_name: Optional[str] = None,
        prompt: Optional[str] = None,
        model: Optional[ModelName] = None,
        threshold: float = 0.0,
    ):
        """Validator describes a single Evaluator and an optional guard value (threshold) used within an Objective

        Note that at most one of evaluator_id and evaluator_name must be provided.

        Args:

          evaluator_id: Skill ID of the evaluator to be used within
            the validator. It must have is_evaluator=True.

          evaluator_name: Name of the evaluator to be used within the validator.


          prompt: Prompt to be used. This is only used if
            evaluator_id/evaluator_name are not supplied.

          model: Model to be used with the prompt. This is only used if
            evaluator_id/evaluator_name are not supplied.

          threshold: The result from the evaluator below which the
            validator will fail. If threshold is 0.0, the Validator
            instance does not validate the results, only providing the
            evaluator values for later study.
        """
        if evaluator_id:
            if evaluator_name or prompt:
                raise ValueError("If evaluator_id is specified, evaluator_name and prompt are not used")
        elif not evaluator_name:
            raise ValueError("evaluator_id or evaluator_name must be provided")
        if prompt is None and model is not None:
            raise ValueError("Model can be only provided alongside prompt")
        self.evaluator_id = evaluator_id
        self.evaluator_name = evaluator_name
        self.model = model
        self.prompt = prompt
        self.threshold = threshold

    def _to_request(self, skills: Skills) -> SkillValidatorRequest:
        if not self.evaluator_id:
            # Iterate through existing skills with matching evaluator_name and that are accessible to the user
            for skill in skills.list(self.prompt, name=self.evaluator_name, only_evaluators=True):
                if self.prompt is not None and self.prompt != skill.prompt:
                    continue
                self.evaluator_id = skill.id
                break
            else:
                # Implicitly create the evaluator
                self.evaluator_id = skills.create(
                    self.prompt or "", model=self.model, name=self.evaluator_name, is_evaluator=True
                ).id
        return SkillValidatorRequest(
            evaluator_id=self.evaluator_id, evaluator_name=self.evaluator_name, threshold=self.threshold
        )


# TODO: Deprecate later, drop in 1.0
def PredicateValidator(evaluator_id: str, threshold: float) -> Validator:  # noqa: N802
    """Deprecated name for Validator; please use Validator."""
    return Validator(evaluator_id=evaluator_id, threshold=threshold)



from dataclasses import dataclass
from typing import List, Optional

from root.utils import iterate_cursor_list


def test_iterate_cursor_list():
    test_data = list(range(1000))

    @dataclass
    class DummyPaginationResult:
        next: Optional[str]
        results: List[int]

    calls = 0
    page_size_max = 42

    def dummy_partial(*, cursor, page_size):
        nonlocal calls
        calls += 1
        index = 0
        page_size = min(page_size_max, page_size)
        if cursor:
            index = int(cursor[1:])
        next_cursor = None
        next_index = index + page_size
        if next_index < len(test_data):
            next_cursor = "x" + str(next_index)
        return DummyPaginationResult(next=next_cursor, results=test_data[index:next_index])

    assert list(iterate_cursor_list(dummy_partial, limit=0)) == test_data[:0]
    assert calls == 0

    assert list(iterate_cursor_list(dummy_partial, limit=12)) == test_data[:12]
    assert calls == 1

    # Ensure that page size aligned fetches still call backend only correct number of times
    calls = 0
    limit = page_size_max * 2
    assert list(iterate_cursor_list(dummy_partial, limit=limit)) == test_data[:limit]
    assert calls == 2

    # Ensure that those that do not align make the extra fetch for partials
    calls = 0
    limit = 123
    assert list(iterate_cursor_list(dummy_partial, limit=limit)) == test_data[:limit]
    assert calls == limit // page_size_max + 1

    # Ensure that getting more than what is available works correctly too
    calls = 0
    assert list(iterate_cursor_list(dummy_partial, limit=1234)) == test_data
    assert calls == 1000 // page_size_max + 1



# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool
from typing_extensions import Annotated, Self

from root.generated.openapi_client.models.skill_validator_request import SkillValidatorRequest
from root.generated.openapi_client.models.status_enum import StatusEnum


class PatchedObjectiveRequest(BaseModel):
    """
    PatchedObjectiveRequest
    """  # noqa: E501

    intent: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=10000)]] = None
    status: Optional[StatusEnum] = None
    test_set: Optional[List[List[Annotated[str, Field(min_length=1, strict=True)]]]] = None
    validators: Optional[List[SkillValidatorRequest]] = None
    force_create: Optional[StrictBool] = Field(
        default=None, description="Force creation of a new objective. Applies only to PUT requests."
    )
    __properties: ClassVar[List[str]] = ["intent", "status", "test_set", "validators", "force_create"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PatchedObjectiveRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in validators (list)
        _items = []
        if self.validators:
            for _item in self.validators:
                if _item:
                    _items.append(_item.to_dict())
            _dict["validators"] = _items
        # set to None if test_set (nullable) is None
        # and model_fields_set contains the field
        if self.test_set is None and "test_set" in self.model_fields_set:
            _dict["test_set"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PatchedObjectiveRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "intent": obj.get("intent"),
                "status": obj.get("status"),
                "test_set": obj.get("test_set"),
                "validators": [SkillValidatorRequest.from_dict(_item) for _item in obj["validators"]]
                if obj.get("validators") is not None
                else None,
                "force_create": obj.get("force_create"),
            }
        )
        return _obj



from abc import ABC
from typing import Dict, Literal, Optional, Union

from root.generated.openapi_client.models.data_loader_type_enum import DataLoaderTypeEnum as _DataLoaderTypeEnum

_EngineType = Literal["google", "bing"]
_ParameterDict = Dict[str, Union[str, Dict[str, str], None]]


class _DataLoaderBase(ABC):
    type: _DataLoaderTypeEnum
    name: str

    def _get_parameter_dict(self) -> _ParameterDict:
        raise NotImplementedError


class WebPageDataLoader(_DataLoaderBase):
    """Data loader which retrieves data from a web page

    Args:
      name: A name assigned to the data loader. The data loader name must be included in the Skill prompt variables.
      url: The URL to retrieve the data from

        Example:

          `"http://example.com/something"`
    """

    type = _DataLoaderTypeEnum.WEBPAGE

    def __init__(self, name: str, url: str):
        self.name = name
        self.url = url

    def _get_parameter_dict(self) -> _ParameterDict:
        return {"url": self.url}


class WebSearchDataLoader(_DataLoaderBase):
    """Data loader which retrieves data from web search

    Args:
      name: A name assigned to the data loader. The data loader name must be included in the Skill prompt variables.
      engine: Search engine to use - `"google"` and `"bing"` are supported.
      search_terms: What to search for in the search engine

    """

    type = _DataLoaderTypeEnum.WEBSEARCH

    def __init__(self, name: str, engine: _EngineType, search_terms: str):
        self.name = name
        self.engine = self.validate_engine(engine)
        self.search_terms = search_terms

    def validate_engine(self, engine: str) -> str:
        """Validate that the given search engine is valid.
        ValueError is raised if it is not."""
        if engine not in ["google", "bing"]:
            raise ValueError("Engine must be 'google' or 'bing'")
        return engine

    def _get_parameter_dict(self) -> _ParameterDict:
        return {"engine": self.engine, "search_terms": self.search_terms}


class SqlDataLoader(_DataLoaderBase):
    """Data loader which retrieves data from SQL

    Args:

      name: A name assigned to the data loader. The data loader name must be included in the Skill prompt variables.

      connection_string: Connection string to use - currently it supports
        only PostgreSQL

        Example:

          `"host='localhost' dbname='my_database' user='postgres' password='secret'"`

      query: The SQL query to feed to SQL.

        Example:

          `"SELECT name FROM users"`
    """

    type = _DataLoaderTypeEnum.SQL

    def __init__(self, name: str, connection_string: str, query: str):
        self.name = name
        self.query = query
        self.connection_string = connection_string

    def _get_parameter_dict(self) -> _ParameterDict:
        return {"connection_string": self.connection_string, "query": self.query}


class ApiCallDataLoader(_DataLoaderBase):
    """Data loader which retrieves data from an URL

    Args:

      name: A name assigned to the data loader. The data loader name must be included in the Skill prompt variables.

      endpoint: URL to connect to. Request type is implicitly GET. Example: `"http://example.com/something"`

      headers: Optional set of headers to specify for the request: Example: `{"X-Accept": "application/json"}`
    """

    type = _DataLoaderTypeEnum.API_CALL

    def __init__(self, name: str, endpoint: str, headers: Optional[Dict[str, str]]):
        self.name = name
        self.endpoint = endpoint
        self.headers = headers

    def _get_parameter_dict(self) -> _ParameterDict:
        return {"endpoint": self.endpoint, "headers": self.headers}


DataLoader = Union[WebPageDataLoader, WebSearchDataLoader, SqlDataLoader, ApiCallDataLoader]



from openai import OpenAI
from root import RootSignals
from root.validators import Validator

# Connect to the Root Signals API
rs_client = RootSignals()

model = "gpt-3.5-turbo"
skill = rs_client.skills.create(
    name="My chatbot",
    intent="Simple Q&A chatbot",
    system_message="You are a helpful assistant.",
    model=model,
    validators=[Validator(evaluator_name="Truthfulness", threshold=0.8)],
)

# Start chatting with the skill
client = OpenAI(base_url=skill.openai_base_url, api_key=rs_client.api_key)
messages = [
    {"role": "user", "content": "Why is the sky blue?"},
]
completion = client.chat.completions.create(
    model=model, messages=messages, stream=True
)
for chunk in completion:
    print(chunk.choices[0].delta.content)
# The sky appears blue because of the way sunlight interacts ...



from root import RootSignals

# Connect to the Root Signals API
client = RootSignals()

# Create a skill
skill = client.skills.create(
    """
    Classify this text into one of the following: {{categories}}
    Text: {{text}}
    """
)

# Execute it
response = skill.run(
    {
        "text": "The expectation for rate cuts has been steadily declining.",
        "categories": "Finance, Sports, Politics",
    }
)

print(response)

# "llm_output": "Finance",
# "validation": Validation(is_valid=True, validator_results=[]),
# "model": "gpt-3.5-turbo",
# "execution_log_id": "9b3c713d-7bdc-4f7d-a85c-ed7d92ff4a56",
# "rendered_prompt": "Classify this text into ...",
# "cost": 5.6e-05,



from openai import OpenAI
from root import RootSignals
from root.validators import Validator

# Connect to the Root Signals API
rs_client = RootSignals()

model = "gpt-3.5-turbo"
another_model = "gpt-4"

skill = rs_client.skills.create(
    name="My chatbot",
    intent="Simple Q&A chatbot",
    system_message="You are a helpful assistant.",
    model=model,
    fallback_models=[another_model],
    validators=[Validator(evaluator_name="Truthfulness", threshold=0.8)],
)

# Start chatting with the skill (non-streaming)
client = OpenAI(base_url=skill.openai_base_url, api_key=rs_client.api_key)

messages = [
    # {"role": "system", "content": "You are a helpful assistant."},
    # ^ implicit in skill
    {"role": "user", "content": "Why is the sky blue?"},
]
completion = client.chat.completions.create(model=model, messages=messages)
print(completion.choices[0].message.content)
# The sky appears blue because of the way sunlight interacts ...


# We can use either the model, or one of the fallback models defined for the
# skill. We will use the fallback model here.
messages = [
    {"role": "user", "content": "Why is the sky blue?"},
]
completion = client.chat.completions.create(
    model=another_model, messages=messages
)
print(completion.choices[0].message.content)
# The sky appears blue because of the way sunlight interacts ...


# We can get the full execution details, including the validation results
log = rs_client.execution_logs.get(log_id=completion.id)
print(log.validation_results)
# [
#   evaluator_name: Truthfulness
#   result: 0.9
#   is_valid: true
#   ...
# ]



# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool
from typing_extensions import Annotated, Self

from root.generated.openapi_client.models.data_loader_request import DataLoaderRequest
from root.generated.openapi_client.models.input_variable_request import InputVariableRequest
from root.generated.openapi_client.models.objective_request import ObjectiveRequest
from root.generated.openapi_client.models.reference_variable_request import ReferenceVariableRequest


class SkillTestInputRequest(BaseModel):
    """
    SkillTestInputRequest
    """  # noqa: E501

    test_data: Optional[List[List[Annotated[str, Field(min_length=1, strict=True)]]]] = None
    test_dataset_id: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    prompt: Annotated[str, Field(min_length=1, strict=True)]
    reference_variables: Optional[List[ReferenceVariableRequest]] = None
    data_loaders: Optional[List[DataLoaderRequest]] = None
    input_variables: Optional[List[InputVariableRequest]] = None
    models: Optional[List[Annotated[str, Field(min_length=1, strict=True)]]] = None
    name: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    pii_filter: Optional[StrictBool] = False
    objective: Optional[ObjectiveRequest] = None
    is_evaluator: Optional[StrictBool] = False
    __properties: ClassVar[List[str]] = [
        "test_data",
        "test_dataset_id",
        "prompt",
        "reference_variables",
        "data_loaders",
        "input_variables",
        "models",
        "name",
        "pii_filter",
        "objective",
        "is_evaluator",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SkillTestInputRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in reference_variables (list)
        _items = []
        if self.reference_variables:
            for _item in self.reference_variables:
                if _item:
                    _items.append(_item.to_dict())
            _dict["reference_variables"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in data_loaders (list)
        _items = []
        if self.data_loaders:
            for _item in self.data_loaders:
                if _item:
                    _items.append(_item.to_dict())
            _dict["data_loaders"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in input_variables (list)
        _items = []
        if self.input_variables:
            for _item in self.input_variables:
                if _item:
                    _items.append(_item.to_dict())
            _dict["input_variables"] = _items
        # override the default output from pydantic by calling `to_dict()` of objective
        if self.objective:
            _dict["objective"] = self.objective.to_dict()
        # set to None if test_data (nullable) is None
        # and model_fields_set contains the field
        if self.test_data is None and "test_data" in self.model_fields_set:
            _dict["test_data"] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        # set to None if objective (nullable) is None
        # and model_fields_set contains the field
        if self.objective is None and "objective" in self.model_fields_set:
            _dict["objective"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SkillTestInputRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "test_data": obj.get("test_data"),
                "test_dataset_id": obj.get("test_dataset_id"),
                "prompt": obj.get("prompt"),
                "reference_variables": [
                    ReferenceVariableRequest.from_dict(_item) for _item in obj["reference_variables"]
                ]
                if obj.get("reference_variables") is not None
                else None,
                "data_loaders": [DataLoaderRequest.from_dict(_item) for _item in obj["data_loaders"]]
                if obj.get("data_loaders") is not None
                else None,
                "input_variables": [InputVariableRequest.from_dict(_item) for _item in obj["input_variables"]]
                if obj.get("input_variables") is not None
                else None,
                "models": obj.get("models"),
                "name": obj.get("name"),
                "pii_filter": obj.get("pii_filter") if obj.get("pii_filter") is not None else False,
                "objective": ObjectiveRequest.from_dict(obj["objective"]) if obj.get("objective") is not None else None,
                "is_evaluator": obj.get("is_evaluator") if obj.get("is_evaluator") is not None else False,
            }
        )
        return _obj



from root import RootSignals

# Connect to the Root Signals API
client = RootSignals()

# Create a skill
skill = client.skills.create(
    name="My text classifier",
    intent="To classify text into arbitrary categories based on semantics",
    prompt="""
    Classify this text into one of the following: {{categories}}
    Text: {{text}}
    """,
    model="gpt-4",
)

# Execute
response = skill.run(
    {
        "text": "The expectation for rate cuts has been steadily declining.",
        "categories": "Finance, Sports, Politics",
    }
)
print(response)
# "llm_output": "Finance",
# "validation": Validation(is_valid=True, validator_results=[]),
# "model": "gpt-4",
# "execution_log_id": "1181e790-7b87-457f-a2cb-6b1dfc1eddf4",
# "rendered_prompt": "Classify this text into ...",
# "cost": 0.00093,

# We can retrieve the skill by id
skill_2 = client.skills.get(skill_id=skill.id)
response = skill_2.run(
    {
        "text": "The expectation for rate cuts has been steadily declining.",
        "categories": "Finance, Sports, Politics",
    }
)

# We can also retrieve it by name
# (the list result is an iterator, so we just take first one)
#
# The name is not an unique identifier. Consequently, the .run method is not
# intentionally available. However, you can circumvent this restriction if you
# wish by using:
skill_3 = next(client.skills.list(name="My text classifier"))
response = client.skills.run(
    skill_3.id,
    {
        "text": "The expectation for rate cuts has been steadily declining.",
        "categories": "Finance, Sports, Politics",
    },
)



from root import RootSignals

# Connect to the Root Signals API
client = RootSignals()

evaluator_id = list(
    client.skills.list(name="Truthfulness", only_evaluators=True)
)[0].id

result = client.evaluators.run(
    evaluator_id,
    request="Return a recipe for a sauce.",
    response="""Quarter peppers and slice onion.
    I used the seeds and ribs of the peppers for optimal heat.
    Sautee in a small sauce pan with oil on medium heat.
    Keep a lid on it but stir often.""",
    contexts=[
        "This is a cookbook with many cookies recipes such as: "
        "1) Recipe for a tomato sauce",
    ],
)
print(result.score)
# 0.0



# coding: utf-8

# flake8: noqa
"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

# import models into model package
from root.generated.openapi_client.models.chat import Chat
from root.generated.openapi_client.models.chat_create_request import ChatCreateRequest
from root.generated.openapi_client.models.chat_detail import ChatDetail
from root.generated.openapi_client.models.chat_execution_request_request import ChatExecutionRequestRequest
from root.generated.openapi_client.models.chat_execution_result import ChatExecutionResult
from root.generated.openapi_client.models.chat_message import ChatMessage
from root.generated.openapi_client.models.data_loader import DataLoader
from root.generated.openapi_client.models.data_loader_request import DataLoaderRequest
from root.generated.openapi_client.models.data_loader_type_enum import DataLoaderTypeEnum
from root.generated.openapi_client.models.data_set_create import DataSetCreate
from root.generated.openapi_client.models.data_set_create_request import DataSetCreateRequest
from root.generated.openapi_client.models.data_set_list import DataSetList
from root.generated.openapi_client.models.data_set_type import DataSetType
from root.generated.openapi_client.models.evaluator_demonstrations import EvaluatorDemonstrations
from root.generated.openapi_client.models.evaluator_demonstrations_request import EvaluatorDemonstrationsRequest
from root.generated.openapi_client.models.evaluator_execution_function_parameter_property_request import (
    EvaluatorExecutionFunctionParameterPropertyRequest,
)
from root.generated.openapi_client.models.evaluator_execution_function_parameter_request import (
    EvaluatorExecutionFunctionParameterRequest,
)
from root.generated.openapi_client.models.evaluator_execution_function_parameter_type_enum import (
    EvaluatorExecutionFunctionParameterTypeEnum,
)
from root.generated.openapi_client.models.evaluator_execution_function_request import EvaluatorExecutionFunctionRequest
from root.generated.openapi_client.models.evaluator_execution_functions_request import (
    EvaluatorExecutionFunctionsRequest,
)
from root.generated.openapi_client.models.evaluator_execution_functions_type_enum import (
    EvaluatorExecutionFunctionsTypeEnum,
)
from root.generated.openapi_client.models.evaluator_execution_request import EvaluatorExecutionRequest
from root.generated.openapi_client.models.evaluator_execution_result import EvaluatorExecutionResult
from root.generated.openapi_client.models.execution_log_details import ExecutionLogDetails
from root.generated.openapi_client.models.execution_log_details_objective import ExecutionLogDetailsObjective
from root.generated.openapi_client.models.execution_log_details_skill import ExecutionLogDetailsSkill
from root.generated.openapi_client.models.execution_log_list import ExecutionLogList
from root.generated.openapi_client.models.id import ID
from root.generated.openapi_client.models.input_variable import InputVariable
from root.generated.openapi_client.models.input_variable_request import InputVariableRequest
from root.generated.openapi_client.models.model import Model
from root.generated.openapi_client.models.model_create import ModelCreate
from root.generated.openapi_client.models.model_create_request import ModelCreateRequest
from root.generated.openapi_client.models.model_params import ModelParams
from root.generated.openapi_client.models.model_params_request import ModelParamsRequest
from root.generated.openapi_client.models.nested_objective_list import NestedObjectiveList
from root.generated.openapi_client.models.nested_skill_evaluator import NestedSkillEvaluator
from root.generated.openapi_client.models.nested_skill_evaluator_request import NestedSkillEvaluatorRequest
from root.generated.openapi_client.models.nested_user_details import NestedUserDetails
from root.generated.openapi_client.models.objective import Objective
from root.generated.openapi_client.models.objective_execution_request import ObjectiveExecutionRequest
from root.generated.openapi_client.models.objective_list import ObjectiveList
from root.generated.openapi_client.models.objective_request import ObjectiveRequest
from root.generated.openapi_client.models.open_ai_chat_completion_request import OpenAIChatCompletionRequest
from root.generated.openapi_client.models.open_ai_message_request import OpenAIMessageRequest
from root.generated.openapi_client.models.paginated_chat_list import PaginatedChatList
from root.generated.openapi_client.models.paginated_data_set_list_list import PaginatedDataSetListList
from root.generated.openapi_client.models.paginated_execution_log_list_list import PaginatedExecutionLogListList
from root.generated.openapi_client.models.paginated_model_list import PaginatedModelList
from root.generated.openapi_client.models.paginated_objective_list_list import PaginatedObjectiveListList
from root.generated.openapi_client.models.paginated_skill_list import PaginatedSkillList
from root.generated.openapi_client.models.paginated_skill_list_output_list import PaginatedSkillListOutputList
from root.generated.openapi_client.models.patched_objective_request import PatchedObjectiveRequest
from root.generated.openapi_client.models.patched_skill_request import PatchedSkillRequest
from root.generated.openapi_client.models.provider import Provider
from root.generated.openapi_client.models.reference_variable import ReferenceVariable
from root.generated.openapi_client.models.reference_variable_dynamic_datasets_request import (
    ReferenceVariableDynamicDatasetsRequest,
)
from root.generated.openapi_client.models.reference_variable_request import ReferenceVariableRequest
from root.generated.openapi_client.models.role_enum import RoleEnum
from root.generated.openapi_client.models.skill import Skill
from root.generated.openapi_client.models.skill_execution_request import SkillExecutionRequest
from root.generated.openapi_client.models.skill_execution_result import SkillExecutionResult
from root.generated.openapi_client.models.skill_execution_validator_result import SkillExecutionValidatorResult
from root.generated.openapi_client.models.skill_list_output import SkillListOutput
from root.generated.openapi_client.models.skill_request import SkillRequest
from root.generated.openapi_client.models.skill_test_data_request import SkillTestDataRequest
from root.generated.openapi_client.models.skill_test_input_request import SkillTestInputRequest
from root.generated.openapi_client.models.skill_test_output import SkillTestOutput
from root.generated.openapi_client.models.skill_type_enum import SkillTypeEnum
from root.generated.openapi_client.models.skill_validator import SkillValidator
from root.generated.openapi_client.models.skill_validator_execution_request import SkillValidatorExecutionRequest
from root.generated.openapi_client.models.skill_validator_request import SkillValidatorRequest
from root.generated.openapi_client.models.status_change import StatusChange
from root.generated.openapi_client.models.status_change_request import StatusChangeRequest
from root.generated.openapi_client.models.status_change_status_enum import StatusChangeStatusEnum
from root.generated.openapi_client.models.status_enum import StatusEnum
from root.generated.openapi_client.models.validation import Validation
from root.generated.openapi_client.models.validation_result_status import ValidationResultStatus
from root.generated.openapi_client.models.validator_execution_result import ValidatorExecutionResult
from root.generated.openapi_client.models.validator_result import ValidatorResult
from root.generated.openapi_client.models.variables_request import VariablesRequest



# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated, Self

from root.generated.openapi_client.models.model_params_request import ModelParamsRequest
from root.generated.openapi_client.models.reference_variable_dynamic_datasets_request import (
    ReferenceVariableDynamicDatasetsRequest,
)


class SkillExecutionRequest(BaseModel):
    """
    SkillExecutionRequest
    """  # noqa: E501

    skill_version_id: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    variables: Optional[Dict[str, Annotated[str, Field(min_length=1, strict=True)]]] = None
    model_params: Optional[ModelParamsRequest] = None
    dynamic_datasets: Optional[List[ReferenceVariableDynamicDatasetsRequest]] = Field(
        default=None, description="Specify the dataset for a reference variable."
    )
    language: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=255)]] = "en"
    __properties: ClassVar[List[str]] = [
        "skill_version_id",
        "variables",
        "model_params",
        "dynamic_datasets",
        "language",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SkillExecutionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of model_params
        if self.model_params:
            _dict["model_params"] = self.model_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in dynamic_datasets (list)
        _items = []
        if self.dynamic_datasets:
            for _item in self.dynamic_datasets:
                if _item:
                    _items.append(_item.to_dict())
            _dict["dynamic_datasets"] = _items
        # set to None if skill_version_id (nullable) is None
        # and model_fields_set contains the field
        if self.skill_version_id is None and "skill_version_id" in self.model_fields_set:
            _dict["skill_version_id"] = None

        # set to None if dynamic_datasets (nullable) is None
        # and model_fields_set contains the field
        if self.dynamic_datasets is None and "dynamic_datasets" in self.model_fields_set:
            _dict["dynamic_datasets"] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict["language"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SkillExecutionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "skill_version_id": obj.get("skill_version_id"),
                "variables": obj.get("variables"),
                "model_params": ModelParamsRequest.from_dict(obj["model_params"])
                if obj.get("model_params") is not None
                else None,
                "dynamic_datasets": [
                    ReferenceVariableDynamicDatasetsRequest.from_dict(_item) for _item in obj["dynamic_datasets"]
                ]
                if obj.get("dynamic_datasets") is not None
                else None,
                "language": obj.get("language") if obj.get("language") is not None else "en",
            }
        )
        return _obj
